% Copyright 2006 by Till Tantau
%
% This file may be distributed and/or modified
%
% 1. under the LaTeX Project Public License and/or
% 2. under the GNU Public License.
%
% See the file doc/generic/pgf/licenses/LICENSE for more details.

\ProvidesFileRCS $Header: /cvsroot/pgf/pgf/generic/pgf/systemlayer/pgfsys-dvipdfm.def,v 1.21 2013/07/16 16:03:13 tantau Exp $

% Driver commands for dvipdfm

%
% Load common pdf and pdf in dvi commands:
%
\input pgfsys-common-pdf.def
\input pgfsys-common-pdf-via-dvi.def

%
% dvipdfm-specific stuff:
%

\def\pgfsys@invoke#1{\special{pdf: content Q #1 q}} % ugly, but necessary...

\def\pgfsys@papersize#1#2{%
%  \expandafter\gdef\expandafter\pgfsys@atbegindocument%
%  \expandafter{\pgfsys@atbegindocument
    \special{papersize=#1,#2}%
%  }
}


%
% Position tracking (needs pdftex >= 1.40.0-alpha-20051205 in dvi mode)
%
\ifx\pdftexversion\@undefined
\else
  \ifnum\pdftexversion>139
    \pgf@sys@pdf@dvi@tracker
  \fi
\fi

\endinput

%%% Local Variables: 
%%% mode: latex
%%% End: 
